__version__ = '0.0.2'
__expected_gendiscal_version__ = 'GenDisCal v1.3.1'
__gendiscal_binary_url__ = 'https://github.com/LM-UGent/GenDisCal/releases/download/1.3.1/GenDisCal_v1_3_1_Linux.tar.gz'

from .GenDisCal import GenDisCal
from .GenDisCalTree import GenDisCalTree
